(* Import libvirt code into CIL.
 * $Id: cilimportvirt.ml,v 1.2 2007/07/07 13:41:23 rjones Exp $
 *)

open Printf
open Cil

let input_file filename =
  let chan = open_in filename in
  let lines = ref [] in
  try while true; do lines := input_line chan :: !lines done; []
  with
    End_of_file -> close_in chan; List.rev !lines

let () =
  (* Read the list of files from "libvirt-files". *)
  let files = input_file "libvirt-files" in

  (* Read the list of symbols in the external interface from "libvirt-syms". *)
  let exported_symbols = input_file "libvirt-syms" in

  (* Load each input file. *)
  let files =
    List.map (
      fun filename ->
	printf "Importing %s ...\n%!" filename;
	(* Why does parse return a continuation? *)
	let f = Frontc.parse filename in
	f ()
    ) files in

  (* Merge them. *)
  let file = Mergecil.merge files "test" in

  (* Do control-flow-graph analysis. *)
  Cfg.computeFileCFG file;

  (* Go over the internal CIL structure and print some facts. *)
  printf "CIL has loaded the library files and merged them.\n\n";

  printf "Merged file has %d globals ...\n\n" (List.length file.globals);
  List.iter (
    function
    | GFun (fundec, loc) ->		(* function definition *)
	let name = fundec.svar.vname in
	let is_exported = List.mem name exported_symbols in

	if is_exported then (
	  printf "%s:%d: %s exported function with has %d arg(s)\n"
	    loc.file loc.line name (List.length fundec.sformals);

	  (* Print the function's complete type. *)
	  print_string "  ";
	  let doc = d_type () fundec.svar.vtype in
	  Pretty.fprint stdout ~width:80 doc;
	  print_newline ()
	);

    | _ -> ()
  ) file.globals;
